import marshmallow_objects as mm

from bitbucket_webhooks_router import base_schemas


class ApprovalInfo(mm.Model):
    date = mm.fields.DateTime()
    user = mm.NestedModel(base_schemas.User)


class RepoPush(mm.Model):
    class Meta:
        unknown = mm.EXCLUDE

    actor = mm.NestedModel(base_schemas.User)
    repository = mm.NestedModel(base_schemas.Repository)


# Pull request events
# https://confluence.atlassian.com/bitbucket/event-payloads-740262817.html#EventPayloads-Pullrequestevents


class PullRequestCreated(mm.Model):
    class Meta:
        unknown = mm.EXCLUDE

    actor = mm.NestedModel(base_schemas.User)
    repository = mm.NestedModel(base_schemas.Repository)
    pullrequest = mm.NestedModel(base_schemas.PullRequest)


class PullRequestUpdated(PullRequestCreated):
    pass


class PullRequestMerged(PullRequestCreated):
    pass


class PullRequestDeclined(PullRequestCreated):
    pass


class PullRequestApproved(mm.Model):
    class Meta:
        unknown = mm.EXCLUDE

    actor = mm.NestedModel(base_schemas.User)
    repository = mm.NestedModel(base_schemas.Repository)
    pullrequest = mm.NestedModel(base_schemas.PullRequest)
    approval = mm.NestedModel(ApprovalInfo)


class PullRequestUnapproved(PullRequestApproved):
    pass



from bitbucket_webhooks_router import _exceptions
from bitbucket_webhooks_router import _handlers

_HANDLER_MAP = {
    handler.event_key: handler for handler in _handlers.EventHandler.__subclasses__()
}


def route(event_key: str, event_payload: dict) -> list:
    handler = _HANDLER_MAP.get(event_key)
    if not handler:
        raise _exceptions.NoHandlerError
    event = handler.schema().load(event_payload)
    return [method(event) for method in handler.decorator.methods]



from abc import ABCMeta
from abc import abstractmethod

import marshmallow_objects as mm

from bitbucket_webhooks_router import decorators
from bitbucket_webhooks_router import event_schemas


class EventHandler(metaclass=ABCMeta):
    @property
    @abstractmethod
    def event_key(self) -> str:
        raise NotImplementedError

    @property
    @abstractmethod
    def schema(self) -> mm.Model:
        raise NotImplementedError

    @property
    @abstractmethod
    def decorator(self) -> decorators.BaseHandler:
        raise NotImplementedError


class RepoPushHandler(EventHandler):
    event_key = "repo:push"
    schema = event_schemas.RepoPush
    decorator = decorators.handle_repo_push


class PullRequestApprovedHandler(EventHandler):
    event_key = "pullrequest:approved"
    schema = event_schemas.PullRequestApproved
    decorator = decorators.handle_pr_approved


class PullRequestCreatedHandler(EventHandler):
    event_key = "pullrequest:created"
    schema = event_schemas.PullRequestCreated
    decorator = decorators.handle_pr_created


class PullRequestUpdatedHandler(EventHandler):
    event_key = "pullrequest:updated"
    schema = event_schemas.PullRequestUpdated
    decorator = decorators.handle_pr_updated


class PullRequestMergedHandler(EventHandler):
    event_key = "pullrequest:fulfilled"
    schema = event_schemas.PullRequestMerged
    decorator = decorators.handle_pr_merged


class PullRequestDeclinedHandler(EventHandler):
    event_key = "pullrequest:rejected"
    schema = event_schemas.PullRequestDeclined
    decorator = decorators.handle_pr_declined



import marshmallow_objects as mm


class User(mm.Model):
    """
    https://confluence.atlassian.com/bitbucket/event-payloads-740262817.html#EventPayloads-entity_user
    """

    class Meta:
        unknown = mm.EXCLUDE

    type = mm.fields.String()
    username = mm.fields.String()
    nickname = mm.fields.String()
    display_name = mm.fields.String()
    uuid = mm.fields.String()


class Project(mm.Model):
    """
    """

    class Meta:
        unknown = mm.EXCLUDE

    name = mm.fields.String()
    uuid = mm.fields.String()
    key = mm.fields.String()


class Repository(mm.Model):
    """
    """

    class Meta:
        unknown = mm.EXCLUDE

    name = mm.fields.String()
    full_name = mm.fields.String()
    website = mm.fields.String(allow_none=True)
    uuid = mm.fields.String()
    owner = mm.NestedModel(User)
    project = mm.NestedModel(Project)
    is_private = mm.fields.Boolean()


class PullRequest(mm.Model):
    """
    """

    class Meta:
        unknown = mm.EXCLUDE

    id = mm.fields.Int()
    title = mm.fields.String()
    description = mm.fields.String()
    # TODO: Change this to enum.
    state = mm.fields.String()
    author = mm.NestedModel(User)
    participants = mm.fields.List(mm.NestedModel(User))
    reviewers = mm.fields.List(mm.NestedModel(User))
    closed_by = mm.NestedModel(User, allow_none=True)
    reason = mm.fields.String()
    created_on = mm.fields.DateTime()
    updated_on = mm.fields.DateTime()



from os import path

from setuptools import find_namespace_packages
from setuptools import setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="bb-hooks-router",
    version="0.0.4",
    description="Routes bitbucket webhook API event payloads to easily consumable decorators.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mukund-murali/bitbucket-webhooks-router",
    author="Mukund Muralikrishnan",
    author_email="mukund.muralikrishnan@gmail.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    keywords="bitbucket webhooks development",
    packages=find_namespace_packages(include=["bitbucket_webhooks_router"]),
    python_requires=">=3.5",
    install_requires=["marshmallow-objects>=2.2.2", "marshmallow>=3.0.0"],
    extras_require={"dev": ["pytest", "black", "isort", "flake8", "mypy"]},
    project_urls={
        "Bug Reports": "https://github.com/mukund-murali/bitbucket-webhooks-router/issues",
        "Source": "https://github.com/mukund-murali/bitbucket-webhooks-router",
    },
)



from abc import ABCMeta
from abc import abstractmethod
from typing import Callable
from typing import List


class BaseHandler(metaclass=ABCMeta):
    def __init__(self, method: Callable) -> None:
        type(self).methods.append(method)  # type: ignore

    @property
    @abstractmethod
    def methods(self) -> List[Callable]:
        raise NotImplementedError


class handle_repo_push(BaseHandler):
    methods: List[Callable] = []

    def __init__(self, method: Callable) -> None:
        super(handle_repo_push, self).__init__(method)


class handle_pr_approved(BaseHandler):
    methods: List[Callable] = []

    def __init__(self, method: Callable) -> None:
        super(handle_pr_approved, self).__init__(method)


class handle_pr_created(BaseHandler):
    methods: List[Callable] = []

    def __init__(self, method: Callable) -> None:
        super(handle_pr_created, self).__init__(method)


class handle_pr_updated(BaseHandler):
    methods: List[Callable] = []

    def __init__(self, method: Callable) -> None:
        super(handle_pr_updated, self).__init__(method)


class handle_pr_merged(BaseHandler):
    methods: List[Callable] = []

    def __init__(self, method: Callable) -> None:
        super(handle_pr_merged, self).__init__(method)


class handle_pr_declined(BaseHandler):
    methods: List[Callable] = []

    def __init__(self, method: Callable) -> None:
        super(handle_pr_declined, self).__init__(method)
